@@ -11,11 +11,12 @@ use strict;
 use warnings;
 use Data::Dumper
 
-our $VERSION = '1.72';  ## Current version of this package
+our $VERSION = '1.70';  ## Current version of this package
+use 5.14.0;
 
 =head1 NAME
 
-Pod::Checker - check pod documents for syntax errors
+Pod::Checker, podchecker() - check pod documents for syntax errors
 
 =head1 SYNOPSIS
 
@@ -47,7 +48,7 @@ trigger additional warnings. See L<"Warnings">.
 
 =item B<-quiet> =E<gt> I<val>
 
-If C<val> is true, do not print any errors/warnings.
+If C<num> is true, do not print any errors/warnings.
 
 =back
 
@@ -339,14 +340,6 @@ there were no POD commands at all found in the file.
 
 See L</SYNOPSIS>
 
-=head1 SCRIPTS
-
-The B<podchecker> script that comes with this distribution is a lean wrapper
-around this module. See the online manual with
-
-  podchecker -help
-  podchecker -man
-
 =head1 INTERFACE
 
 While checking, this module collects document properties, e.g. the nodes
@@ -452,7 +445,6 @@ sub new {
     $new->{'_fcode_stack'} = [];    # stack for nested formatting codes
     $new->{'_fcode_pos'} = [];      # stack for position in paragraph of fcodes
     $new->{'_begin_stack'} = [];    # stack for =begins: [line #, target]
-    $new->{'_links'} = [];          # stack for hyperlinks to external entities
     $new->{'_internal_links'} = []; # set of linked-to internal sections
     $new->{'_index'} = [];          # stack for text in X<>s
 
@@ -502,11 +494,8 @@ sub poderror {
     ## Retrieve options
     chomp( my $msg  = ($opts{'-msg'} || '')."@_" );
     my $line = (exists $opts{'-line'}) ? " at line $opts{'-line'}" : '';
-    my $file = ' in file ' . ((exists $opts{'-file'})
-                              ? $opts{'-file'}
-                              : ((defined $self->source_filename)
-                                 ? $self->source_filename
-                                 : "???"));
+    my $file = ' in file ' . ((exists $opts{'-file'}) ?
+                             $opts{'-file'} : $self->source_filename);
     unless (exists $opts{'-severity'}) {
        ## See if can find severity in message prefix
        $opts{'-severity'} = $1  if ( $msg =~ s/^\**\s*([A-Z]{3,}):\s+// );
@@ -597,6 +586,8 @@ Add (if argument specified) and retrieve the index entries (as defined by
 C<XE<lt>E<gt>>) of the current POD. They consist of plain text, each piece
 of whitespace is collapsed to a single blank.
 
+=back
+
 =cut
 
 # set/return index entries of current POD
@@ -616,29 +607,6 @@ sub idx {
 
 ##################################
 
-# add a hyperlink to the list of those of the current POD; returns current
-# list after the addition has been done
-sub hyperlink {
-    my $self = shift;
-    push(@{$self->{'_links'}}, $_[0]);
-    return $_[0];
-}
-
-=item C<$checker-E<gt>hyperlinks()>
-
-Retrieve an array containing the hyperlinks to things outside
-the current POD (as defined by C<LE<lt>E<gt>>).
-
-Each is an instance of a class with the following methods:
-
-=cut
-
-sub hyperlinks {
-    @{shift->{'_links'}};
-}
-
-##################################
-
 # override Pod::Simple's whine() and scream() to use poderror()
 
 # Note:
@@ -697,10 +665,6 @@ sub _check_fcode {
     # Check for an fcode inside another of the same fcode
     # XXX line number is the line of the start of the paragraph that the warning
     # is in, not the line that the warning is on. Fix this
-
-    # Later versions of Pod::Simple forbid nested L<>'s
-    return if $inner eq 'L' && $Pod::Simple::VERSION ge '3.33';
-
     if (grep { $_ eq $inner } @$outers) {
         $self->poderror({ -line => $self->{'_line'},
                           -severity => 'WARNING',
@@ -790,9 +754,7 @@ sub end_head3 { shift->end_head(@_) }
 sub end_head4 { shift->end_head(@_) }
 sub end_head  {
     my $self = shift;
-    my $arg = $self->{'_thispara'};
-    $arg =~ s/\s+$//;
-    $self->{'_head_text'} = $arg;
+    my $arg = $self->{'_head_text'} = $self->{'_thispara'} =~ s/\s+$//r;
     $self->{'_cmds_since_head'} = 0;
     my $h = $self->{'_head_num'};
     $self->node($arg); # remember this node
@@ -829,13 +791,7 @@ sub end_item_text   { shift->end_item('definition') }
 sub end_item {
     my $self = shift;
     my $type = shift;
-    # If there is verbatim text in this item, it will show up as part of
-    # 'paras', and not part of '_thispara'.  If the first para after this is a
-    # verbatim one, it actually will be (part of) the contents for this item.
-    if (   $self->{'_thispara'} eq ''
-        && (  ! @{$self->{'paras'}}
-            ||    $self->{'paras'}[0][0] !~ /Verbatim/i))
-    {
+    if ($self->{'_thispara'} eq '') {
         $self->poderror({ -line => $self->{'_line'},
                           -severity => 'WARNING',
                           -msg => '=item has no contents' });
@@ -847,12 +803,12 @@ sub end_item {
 sub start_for { # =for and =begin directives
     my ($self, $flags) = @_;
     $self->_init_event($flags);
-    push @{$self->{'_begin_stack'}}, [$self->{'_line'}, $flags->{'target'}];
+    push $self->{'_begin_stack'}, [$self->{'_line'}, $flags->{'target'}];
 }
 
 sub end_for {
     my ($self, $flags) = @_;
-    my ($line, $target) = @{pop @{$self->{'_begin_stack'}}};
+    my ($line, $target) = @{pop $self->{'_begin_stack'}};
     if ($flags->{'fake-closer'}) { # meaning Pod::Simple generated this =end
         $self->poderror({ -line => $line,
                           -severity => 'ERROR',
@@ -886,6 +842,8 @@ sub end_Document {
     # know what it means...
 
     for my $link (@{ $self->{'_internal_links'} }) {
+        # XXX What if there is a link to the page by the name
+        # e.g. in Tk::Pod : L<Tk::Pod/"DESCRIPTION">
         my ($name, $line) = @$link;
         unless ( $nodes{$name} ) {
             $self->poderror({ -line => $line,
@@ -919,7 +877,7 @@ sub start_I { shift->start_fcode('I') }
 sub start_S { shift->start_fcode('S') }
 sub start_fcode {
     my ($self, $fcode) = @_;
-    unshift @{$self->{'_fcode_stack'}}, $fcode;
+    unshift $self->{'_fcode_stack'}, $fcode;
 }
 
 sub end_B { shift->end_fcode() }
@@ -929,7 +887,7 @@ sub end_I { shift->end_fcode() }
 sub end_S { shift->end_fcode() }
 sub end_fcode {
     my $self = shift;
-    $self->_check_fcode(shift @{$self->{'_fcode_stack'}}, # current fcode removed
+    $self->_check_fcode(shift $self->{'_fcode_stack'}, # current fcode removed
                         $self->{'_fcode_stack'}); # previous fcodes
 }
 
@@ -937,19 +895,36 @@ sub start_L {
     my ($self, $flags) = @_;
     $self->start_fcode('L');
 
-    my $link = Pod::Checker::Hyperlink->new($flags, $self);
-    if ($link) {
-        if (   $link->type eq 'pod'
-            && $link->node
-                # It's an internal-to-this-page link if no page is given, or
-                # if the given one is to our NAME.
-            && (! $link->page || (   $self->{'_pod_name'}
-                                  && $link->page eq $self->{'_pod_name'})))
-        {
-            push @{ $self->{'_internal_links'} }, [ $link->{'-raw_node'}, $link->line ];
-        }
-        else {
-            $self->hyperlink($link);
+    # We use Pod::Hyperlink to parse the raw L<> contents into the -alttext,
+    # -page, and -node, but we then use Pod::Simple's parsing to override -page
+    # and -node as Pod::Simple handles them better. An example of this is the
+    # handling of leading/trailing whitespace: Pod::Simple correctly interprets
+    # leading/trailing whitespace to mean the contents are a section, whereas
+    # Pod::Hyperlink removes the whitespace and treats the contents as a page.
+    # Note that we also use Pod::Hyperlink to issue some L<> error checks.
+
+}
+
+sub _treat_Ls {
+    my ($self, @stack) = @_;
+
+    my $start_line = $stack[0][1]{'start_line'};
+
+    $self->SUPER::_treat_Ls(@stack);
+
+    while(my $treelet = shift @stack) {
+        for(my $i = 2; $i < @$treelet; ++$i) {
+            # iterate over children of current tree node
+            next unless ref $treelet->[$i];  # text nodes are uninteresting
+            unless($treelet->[$i][0] eq 'L') {
+                unshift @stack, $treelet->[$i]; # recurse
+                next;
+            }
+
+            my $arg = $treelet->[$i][1];
+            if ($arg->{type} eq 'pod' && ! $arg->{to} && $arg->{section}) {
+                push @{ $self->{'_internal_links'} }, [ "$arg->{section}", $start_line ];
+            }
         }
     }
 }
@@ -964,15 +939,15 @@ sub start_X {
     $self->start_fcode('X');
     # keep track of where X<> starts in the paragraph
     # (this is a stack so nested X<>s are handled correctly)
-    push @{$self->{'_fcode_pos'}}, length $self->{'_thispara'};
+    push $self->{'_fcode_pos'}, length $self->{'_thispara'};
 }
 sub end_X {
     my $self = shift;
     # extract contents of X<> and replace with ''
-    my $start = pop @{$self->{'_fcode_pos'}}; # start at the beginning of X<>
+    my $start = pop $self->{'_fcode_pos'}; # start at the beginning of X<>
     my $end = length($self->{'_thispara'}) - $start; # end at end of X<>
     my $x = substr($self->{'_thispara'}, $start, $end, '');
-    if ($x eq "") {
+    unless ($x) {
         $self->poderror({ -line => $self->{'_line'},
                           -severity => 'ERROR',
                           -msg => "An empty X<>" });
@@ -981,97 +956,10 @@ sub end_X {
     $self->end_fcode();
 }
 
-package Pod::Checker::Hyperlink;
 
-# This class is used to represent L<> link structures, so that the individual
-# elements are easily accessible.  It is based on code in Pod::Hyperlink
+1;
 
-sub new {
-    my ($class,
-        $simple_link,   # The link structure returned by Pod::Simple
-        $caller         # The caller class
-    ) = @_;
-
-    my $self = +{};
-    bless $self, $class;
-
-    $self->{'-line'} ||= $caller->{'_line'};
-    $self->{'-type'} ||= $simple_link->{'type'};
-
-    # Force stringification of page and node.  (This expands any E<>.)
-    $self->{'-page'} = exists $simple_link->{'to'} ? "$simple_link->{'to'}" : "";
-    $self->{'-node'} = exists $simple_link->{'section'} ? "$simple_link->{'section'}" : "";
-
-    # Save the unmodified node text, as the .t files are expecting the message
-    # for internal link failures to include it (hence this preserves backward
-    # compatibility).
-    $self->{'-raw_node'} = $self->{'-node'};
-
-    # Remove leading/trailing white space.  Pod::Simple already warns about
-    # these, so if the only error is this, and the link is otherwise correct,
-    # only the Pod::Simple warning will be output, avoiding unnecessary
-    # confusion.
-    $self->{'-page'} =~ s/ ^ \s+ //x;
-    $self->{'-page'} =~ s/ \s+ $ //x;
-
-    $self->{'-node'} =~ s/ ^ \s+ //x;
-    $self->{'-node'} =~ s/ \s+ $ //x;
-
-    # Pod::Simple warns about L<> and L< >, but not L</>
-    if ($self->{'-page'} eq "" && $self->{'-node'} eq "") {
-        $caller->poderror({ -line => $caller->{'_line'},
-                          -severity => 'WARNING',
-                          -msg => 'empty link'});
-        return;
-    }
-
-    return $self;
-}
-
-=item line()
-
-Returns the approximate line number in which the link was encountered
-
-=cut
-
-sub line {
-    return $_[0]->{-line};
-}
-
-=item type()
-
-Returns the type of the link; one of:
-C<"url"> for things like
-C<http://www.foo>, C<"man"> for man pages, or C<"pod">.
-
-=cut
-
-sub type {
-    return  $_[0]->{-type};
-}
-
-=item page()
-
-Returns the linked-to page or url.
-
-=cut
-
-sub page {
-    return $_[0]->{-page};
-}
-
-=item node()
-
-Returns the anchor or node within the linked-to page, or an empty string
-(C<"">) if none appears in the link.
-
-=back
-
-=cut
-
-sub node {
-    return $_[0]->{-node};
-}
+__END__
 
 =head1 AUTHOR
 
@@ -1080,12 +968,8 @@ Please report bugs using L<http://rt.cpan.org>.
 Brad Appleton E<lt>bradapp@enteract.comE<gt> (initial version),
 Marek Rouchal E<lt>marekr@cpan.orgE<gt>,
 Marc Green E<lt>marcgreen@cpan.orgE<gt> (port to Pod::Simple)
-Ricardo Signes E<lt>rjbs@cpan.orgE<gt> (more porting to Pod::Simple)
-Karl Williamson E<lt>khw@cpan.orgE<gt> (more porting to Pod::Simple)
 
 Based on code for B<Pod::Text::pod2text()> written by
 Tom Christiansen E<lt>tchrist@mox.perl.comE<gt>
 
 =cut
-
-1
